package org.greenstone.gsdl3.gs3build.indexers;

import java.util.Iterator;

import org.w3c.dom.Element;

import org.greenstone.gsdl3.gs3build.doctypes.DocumentID;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentInterface;
import org.greenstone.gsdl3.gs3build.doctypes.DocumentList;

public class IndexerManager
{
  IndexerInterface indexers[];
  int size;
  int used;
  DocumentList documentList;

  public static final String outputDir = "outputDir";
  public static final String DEFAULT_LEVEL = "document";
  public static final String DEFAULT_FIELD = "text";

  public IndexerManager(DocumentList documentList)
  { this.indexers  = new IndexerInterface[10];
    this.size      = 10;
    this.used      = 0;
    this.documentList = documentList;
  }

  public void addIndexer(IndexerInterface indexer)
  { this.ensureCapacity(this.used + 1);

    this.indexers[this.used] = indexer;
    this.used ++;
  }

  public void indexDocument(DocumentID docId, DocumentInterface document)
  { if (!document.isIndexed())
      return;
	  
    for (int i = 0; i < this.used; i ++)
    { this.indexers[i].indexDocument(docId, document);
    }
  }

  public void indexDocuments()
  { for (int i = 0; i < this.used; i ++)
    { for (int p = 0; p < this.indexers[i].getNumberOfPasses(); p ++)
      {	
	if (!this.indexers[i].startPass(p)) {
	  continue;
	}

	Iterator iterator = this.documentList.iterator();

	while (iterator.hasNext()) {
	  DocumentInterface document = (DocumentInterface) iterator.next();

	  if (document.isIndexed()) {
	    if (!this.indexers[i].indexDocument(document.getID(), document)) {
	      System.out.println("Ending document " + document.getID());
	    }
	    
	    // note any changes made to this document...
	    if (document.isChanged()) {
	      //System.out.println("Writing document "+document.getID());
	      this.documentList.storeChangedDocument(document);
	    }
	    /**
	    if (p == 0) {
	      System.out.println("Writing document "+document.getID());
	    }
	    */
	  }
	}
	this.indexers[i].endPass(p);
      }
    }
  }

  public void ensureCapacity(int needSize)
  { while (needSize >= this.size)
    { int newSize = this.size * 2;
      IndexerInterface newIndexers [] = new IndexerInterface[newSize];
      System.arraycopy(indexers, 0, newIndexers, 0, this.used);
      this.indexers = newIndexers;
      this.size = newSize;
    }
  }

    public boolean addServiceDescriptions(Element service_class_list) {
	boolean success = true;
	for (int i = 0; i < this.used; i ++) {
	    if (!this.indexers[i].addServiceDescriptions(service_class_list)) {
		success = false;
	    }
	}
	return success;
    }
}

