package org.greenstone.gsdl3.gs3build.util;

import java.util.*;

public class HTMLBlockList
{ Vector blocks;
	String title;
	protected static HTMLTagVector anchorStartTags;
	protected static HTMLTagVector anchorEndTags;
	protected static HTMLTagVector anchorOptEndTags;
	protected static HTMLTagVector headingStartTags;
	protected static HTMLTagVector headingEndTags;

	static
	{	anchorStartTags 	= new HTMLTagVector("a");
		anchorEndTags			= new HTMLTagVector("a,h1,h2,h3,h4,h5,h6,table,hr,p,ul,ol,dl,dt,dd,li,tr,td,th,map");
		anchorOptEndTags 	= new HTMLTagVector("div,center,form");
		headingStartTags 	= new HTMLTagVector("h1,h2,h3,h4,h5,h6");
		headingEndTags	 	= new HTMLTagVector("h1,h2,h3,h4,h5,h6,table,hr,p,ul,ol,dl,dt,dd,li,tr,td,th,map");
	}

	// opening   opentag  incontent	result
	//    T					T					T				*       {if opening, incontent must be false}
	//    T					T					F				T
	//    T					F					T				*       {if opening, incontent must be false}
	//    T					F					F				F
	//    F					T					T				T
	//    F					T					F				F				{if getting an opening tag
	//		F					F					T				T
	//		F					F					F				T
	private boolean inTagList(HTMLTag tag, Vector tags, boolean opening)
	{ int tagref;

		for (tagref = 0; tagref < tags.size(); tagref ++)
		{ if ((tag.xtagName().equals((String) tags.elementAt(tagref))) &&
		      (tag.tagIsOpening() || !opening))
			{ return true;
			}
		}
		return false;
	}

	//
	// Common initialisation; parse the document for text surrounded by
	// pairs of <starttag>....<endtag/starttag> combos
	//
	protected void initialise(HTMLDoc doc, HTMLTagVector starttags, HTMLTagVector endtags, HTMLTagVector optendtags)
	{ boolean intag;
		boolean incontent;
		HTMLCText		text;
		HTMLTag			tag;
		HTMLTag 		starttag;
		HTMLBlock 	block;
		HTMLBlock		docBlock;
		Enumeration	elements;
		Object			element;

		this.blocks = new Vector(1);
		block				= null;
		intag				= false;
		incontent		= false;
		starttag		= null;

		docBlock		= doc.getCodedContent();
		elements		= docBlock.elements();

		while (elements.hasMoreElements())
		{ element	= elements.nextElement();

			// if it's a tag
			if (element instanceof HTMLTag)
			{ tag = (HTMLTag) element;
			
				// end the fragment if getting a closing tag
				if (intag)
				{ if (endtags.matches(tag, false) ||
							(incontent && optendtags.matches(tag, false)))
					{ block.addTag(starttag.endTag());
						intag = false;
						incontent = false;
					}
					else
					{ block.addTag(tag);
					}
				}

				// !intag is checked for explicitly, as it may have been
				// switched off by a non-partner opening tag above: eg:
				//   <H2>A Heading<H3>A Smaller Heading</H3>
				//                ^^^^
				//                here

				// start fragment if the tag is a starter
				if (!intag)
				{ if (starttags.matches(tag, true))
					{ block = new HTMLBlock(tag.startPos(), tag.endPos());
						this.blocks.addElement((Object) block);
						block.addTag(tag);
						starttag = tag;
						intag = true;
						incontent = false;
					}
				}
			}	
			// if the parse returned text
			else
			{ text	= (HTMLCText) element;

				if (intag)
				{ block.addText(text);
					if (!text.nullString())
					{ incontent = true;
					}
				}
			}
		}

		// if a tag wasn't "finished");
		if (intag)
		{ block.addTag(starttag.endTag());
		}

		this.title = doc.urlString();
	}

	protected HTMLBlockList()
	{
	}
	
	//
	// constructor for given vectors of tags: startlist elements always start a new
	// block [ending an open one if necessary]; endlist elements always end an open 
	// block; optendlist elements close a block iff a block has been started, and
	// some non-blank content (ie. some "real" text, not just returns/spaces) has
	// been read since the beginning of the block
	//
	public HTMLBlockList(HTMLDoc doc, Vector starttags, Vector endtags, Vector optendtags)
	{ HTMLTagVector _starttags;
		HTMLTagVector _endtags;
		HTMLTagVector _optendtags;

		_starttags	= new HTMLTagVector(starttags);
		_endtags		= new HTMLTagVector(starttags);
		_optendtags	= new HTMLTagVector(starttags);
		this.initialise(doc, _starttags, _endtags, _optendtags);
	}
	
	//
	// constructor for explicit lists; a tagVector is created for each list;
	// see above for list roles
	//
	public HTMLBlockList(HTMLDoc doc, String startlist, String endlist, String optendlist)
	{ HTMLTagVector starttags;
		HTMLTagVector endtags;
		HTMLTagVector optendtags;

		starttags		= new HTMLTagVector(startlist);
		endtags			= new HTMLTagVector(endlist);
		optendtags	= new HTMLTagVector(optendlist);
		this.initialise(doc, starttags, endtags, optendtags);
	}

	//
	// constructor; get all HTMLBlocks for h{level} through to h1;
	// eg hlevel==3 test h3,h2,h1
	//
	public HTMLBlockList(HTMLDoc doc, int hlevel)
	{ int loop;
		StringBuffer levelstring;
		HTMLTagVector starttags;
		HTMLTagVector endtags;
		HTMLTagVector optendtags;

		levelstring = new StringBuffer("h1");
		for (loop = 2; loop <= hlevel; loop ++)
		{ levelstring.append(",h");
		  levelstring.append(Integer.toString(loop));
		}

		starttags		= new HTMLTagVector(levelstring.toString());
		levelstring.append(",table,hr,p,ul,ol,dl,dt,dd,li,tr,td,th,map");
		endtags			= new HTMLTagVector(levelstring.toString());
		optendtags	= new HTMLTagVector("div,center,form");

		this.initialise(doc, starttags, endtags, optendtags);
	}

	/**
	 * Static method returning list of anchors of a document
	 */
	public static synchronized HTMLBlockList anchorlist(HTMLDoc doc)
	{ HTMLBlockList anchors;
		HTMLTagVector starttags;
		HTMLTagVector endtags;
		HTMLTagVector optendtags;

		anchors = new HTMLBlockList();

		anchors.initialise(doc, anchorStartTags, anchorEndTags, anchorOptEndTags);

		return anchors;
	}

	//
	// return how many tags are in the block list
	//
	public int size()
	{ if (this.blocks == null)
		{ return 0;
		}

		return this.blocks.size();
	}

	/**
	 * @return number of blocks in the block list
	 * @deprecated
	 */
	public int tagCount()
	{	return this.size();
	}

	//
	// return the position of the tag in the document
	//
	public int tagPos(int tag)
	{ HTMLBlock block;

		if (this.blocks == null)
		{ return 0;
		}
		block = (HTMLBlock)	this.blocks.elementAt(tag);
		return block.startPos();
	}

	//
	// return the nth block in the list
	//
	public HTMLBlock tagBlock(int tag)
	{ HTMLBlock block;

		block = (HTMLBlock) this.blocks.elementAt(tag);
		return block;
	}

	//
	// return the index of the next blocklist of the same level
	//
	public int tagNext(int attag)
	{ HTMLBlock block;
		String		headTag;
		int				nextTag;

		block = (HTMLBlock) this.blocks.elementAt(attag);
		headTag = block.headTagName();
		nextTag	= attag+1;

		while (nextTag < this.blocks.size())
		{	block	=	(HTMLBlock) this.blocks.elementAt(nextTag);
			if (block.headTagName().equals(headTag))
			{	break;
			}
			nextTag ++;
		}
		return nextTag;
	}

	//
	// return the number of children at the next heading level
	// 
	public int tagChildren(int fromtag, int level)
	{ int sibling;
		int descendent;
		int children;
		int	atlevel;
		HTMLTag tag;

		// get current tag level; level 6 headings cannot have
		// children
		if (level == 6)
		{ return 0;
		}

		// now, if the next tag at this level is the next tag in
		// the document, then we can't have any children
		if (fromtag > 0)
		{ sibling	= this.tagNext(fromtag-1);
		}
		else
		{	sibling	= this.size();
		}
		if (sibling == fromtag)
		{ children = 0;
		}
		// if the next tag at this level is *not* next, then we have
		// at least one child (ie the next tag in sequence), even if
		// it is not at the next heading level down.
		//
		// if no further children occur at the next level down, we
		// have only one child.  If any further children occur at the
		// next level down, then they are counted, except where it is
		// the next tag in sequence, which is automatically accounted
		// for
		else
		{	children	= 1;
			for (descendent = fromtag+1; descendent < sibling; descendent ++)
			{ tag	= this.tagBlock(descendent).headTag();
				
				if (tag.tagLevel() ==	level + 1)
				{	children ++;
				}
			}
		}
		return children;
	}


	public int tagChildren(int attag)
	{ int sibling;
		int descendent;
		int children;
		int	atlevel;
		HTMLTag tag;

		// get current tag level; level 6 headings cannot have
		// children
		atlevel		=	this.tagBlock(attag).headTag().tagLevel();
		if (atlevel == 6)
		{ return 0;
		}

		// now, if the next tag at this level is the next tag in
		// the document, then we can't have any children
		sibling		= this.tagNext(attag);
		if (sibling == attag + 1)
		{ children = 0;
		}
		// if the next tag at this level is *not* next, then we have
		// at least one child (ie the next tag in sequence), even if
		// it is not at the next heading level down.
		//
		// if no further children occur at the next level down, we
		// have only one child.  If any further children occur at the
		// next level down, then they are counted, except where it is
		// the next tag in sequence, which is automatically accounted
		// for
		else
		{	children	= 1;
			for (descendent = attag + 2; descendent < sibling; descendent ++)
			{ tag	= this.tagBlock(descendent).headTag();
				
				if (tag.tagLevel() ==	atlevel + 1)
				{	children ++;
				}
			}
		}
		return children;
	}

	//
	// write out a structured output onto System.out
	//
  public void sysoutList()
	{ int item;
		HTMLBlock block;

		if (this.blocks == null)
		{ return;
		}

		for (item = 0; item < this.blocks.size(); item ++)
		{ block = (HTMLBlock) this.blocks.elementAt(item);
			System.out.println("  @"+ block.startPos()+":"+block.headTagName() +":"+ block);
		}
	}

	//
	// write out HTMLBlocks as an Enumeration
	//
	public Enumeration elements()
	{ return new HTMLBlockListEnumerator(this);
	}
}

final class HTMLBlockListEnumerator implements Enumeration
{ private HTMLBlockList blocklist;
	private int						member;

	public HTMLBlockListEnumerator(HTMLBlockList blocklist)
	{ this.blocklist	= blocklist;
		this.member			= 0;
	}

	public Object nextElement()
	{ Object element;

		element = this.blocklist.blocks.elementAt(member);
		member ++;
		return element;
	}

	public boolean hasMoreElements()
	{ if (this.blocklist == null || this.member == this.blocklist.blocks.size())
		{ return false;
		}
		return true;
	}
}

